//=============================================================================
//	Step2
//
//	FileName	:	PP100APISampleStep2.cpp
//	Contents	:	Connecting PC that TDBridge is running PP-100 Series API sample application.
//	Language	:	C++
//	Copyright	:	Copyright(C) SEIKO EPSON CORPORATION 2010. All rights reserved.
//	Note		:
//	 1.Outline
//	  Initialize, connect to PC, disconnect from PC, destroy, and output it return value to the console.
//	 2.Process
//	 (1)Initialize PP-100 Series API.
//		PP100_Initialize()
//	 (2)Connect to PC that TDBridge is running.
//		PP100_ConnectServer()
//	 (3)Disconnect from PC that TDBridge is running.
//		PP100_DisConnectServer()
//	 (4)Release PP-100 Series API.
//		PP100_Destroy()
//	CommandLine	:	PP100APISampleStep2
//=============================================================================

//=============================================================================
//	include files (#include)
//=============================================================================
#if defined(_WIN32)
#include <windows.h>
#include <tchar.h>
#include <shfolder.h>
#elif defined(__GNUC__)
#include <dlfcn.h>
#endif//defined(_WIN32),defined(__GNUC__)
#include <stdio.h>
#include "PP100API.h"

//=============================================================================
//	global variables
//=============================================================================
// The full path of PP100API.dll.
// If you changed the installation folder of EPSON TD Bridge, you need to changed the value.
#if defined(_WIN32)
TCHAR g_pPP100APIDLL[] = _T("\\EPSON\\TDBridge\\API\\PP100API.dll");
#elif defined(__GNUC__)
const char g_pPP100APIDLL[] = "/opt/epson/Total Disc Maker/lib/libPP100API.so";
#endif//defined(_WIN32),defined(__GNUC__)
HMODULE g_hPP100APIDLL = NULL;
PP100_INITIALIZE g_pPP100_Initialize = NULL;
PP100_DESTROY g_pPP100_Destroy = NULL;
PP100_CONNECTSERVER g_pPP100_ConnectServer = NULL;
PP100_DISCONNECTSERVER g_pPP100_DisConnectServer = NULL;

//=============================================================================
//	global functions
//=============================================================================
long LoadPP100API(); // Load PP-100 Series API and get function pointers.

//=============================================================================
//	main
//
//	(1)Initialize PP-100 Series API.
//		PP100_Initialize()
//	(2)Connect to PC that TDBridge is running.
//		PP100_ConnectServer()
//	(3)Disconnect from PC that TDBridge is running.
//		PP100_DisConnectServer()
//	(4)Release PP-100 Series API.
//		PP100_Destroy()
//
//	IN:
//		argc		- Number of input parameter.
//		argv		- Pointer to input paraemter.
//	OUT:
//
//	Return: void
//=============================================================================
int main(int argc, char *argv[])
{
	long nRet = LoadPP100API();
	printf("LoadPP100API() returns %ld\n", nRet);
	if(0 != nRet)
	{
		printf("Loading PP-100 Series API failed.\n");
		return nRet;
	}

	//=========================================================================
	// (1)Initialize PP100 Series API.
	//=========================================================================
	nRet = g_pPP100_Initialize();
	printf("PP100_Initialize() returns %ld\n", nRet);
	if(PP100API_SUCCESS != nRet)
	{
		printf("Initializing PP-100 Series API failed.\n");
		return nRet;
	}

	//=========================================================================
	// (2)Connect to PC that TDBridge is running.
	//    This sample sets pHost: 127.0.0.1, pOrderFolder: Orders
	//    i.e. \\127.0.0.1\Orders is full path of monitoring folder.
	//    If pHost is NULL, PP-100 Series API assume that local PC specified, and ignore pOrderFolder parameter.
	//=========================================================================
	unsigned long ulHandle = 0;
#if defined(_WIN32)
	nRet = g_pPP100_ConnectServer(L"127.0.0.1", L"Orders", &ulHandle);
#elif defined(__GNUC__)
	nRet = g_pPP100_ConnectServer(NULL, NULL, &ulHandle);
#endif//defined(_WIN32),defined(__GNUC__)
	printf("PP100_ConnectServer() returns %ld\n", nRet);
	printf("ulHandle is %lu\n", ulHandle);
	if(PP100API_SUCCESS != nRet)
	{
		printf("Connecting to PC that TDBridge is running failed.\n");
		return nRet;
	}

	//=========================================================================
	// (3)Disconnecting from PC that TDBridge is running.
	//=========================================================================
	nRet = g_pPP100_DisConnectServer(ulHandle);
	printf("PP100_DisConnectServer() returns %ld\n", nRet);
	if(PP100API_SUCCESS != nRet)
	{
		printf("Disconnecting from PC that TDBridge is running failed.\n");
		return nRet;
	}

	//=========================================================================
	// (4)Release PP100 Series API.
	//=========================================================================
	nRet = g_pPP100_Destroy();
	printf("PP100_Destroy() returns %ld\n", nRet);
	if(PP100API_SUCCESS != nRet)
	{
		printf("Destroying PP-100 Series API failed.\n");
		return nRet;
	}

	return nRet;
}

//=============================================================================
//	LoadPP100API
//
//	(1)Load PP-100 Series API and get function pointers.
//
//	IN:
//	OUT:
//
//	Return:	0	- success
//			1	- failure
//=============================================================================
long LoadPP100API()
{
	// Load PP100API.dll

#if defined(_WIN32)
	TCHAR tcDLLPath[MAX_PATH];
	HRESULT hResult;
	hResult = SHGetFolderPath(NULL, CSIDL_PROGRAM_FILES, NULL, 0, tcDLLPath);
	if(S_OK != hResult)
	{
		printf("Loading PP-100 Series API failed.\n");
		return 1;
	}

	_tcscat_s(tcDLLPath, _countof(tcDLLPath), g_pPP100APIDLL);

	g_hPP100APIDLL = ::LoadLibrary(tcDLLPath);
	if(NULL == g_hPP100APIDLL)
	{
		printf("Loading PP-100 Series API failed.\n");
		return 1;
	}

	// Get function pointer of PP100_Initialize() and PP100_Destroy()
	g_pPP100_Initialize = (PP100_INITIALIZE)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_INITIALIZE);
	g_pPP100_Destroy = (PP100_DESTROY)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_DESTROY);
	g_pPP100_ConnectServer = (PP100_CONNECTSERVER)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_CONNECTSERVER);
	g_pPP100_DisConnectServer = (PP100_DISCONNECTSERVER)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_DISCONNECTSERVER);
#elif defined(__GNUC__)
	const char *the_path = g_pPP100APIDLL;
	g_hPP100APIDLL = dlopen(the_path, RTLD_LAZY);
	if(NULL == g_hPP100APIDLL)
	{
		printf("Loading PP-100 Series API failed.\n");
		return 1;
	}

	// Get function pointer of PP100_Initialize() and PP100_Destroy()
	g_pPP100_Initialize = (PP100_INITIALIZE)dlsym(g_hPP100APIDLL, FNSTR_PP100_INITIALIZE);
	g_pPP100_Destroy = (PP100_DESTROY)dlsym(g_hPP100APIDLL, FNSTR_PP100_DESTROY);
	g_pPP100_ConnectServer = (PP100_CONNECTSERVER)dlsym(g_hPP100APIDLL, FNSTR_PP100_CONNECTSERVER);
	g_pPP100_DisConnectServer = (PP100_DISCONNECTSERVER)dlsym(g_hPP100APIDLL, FNSTR_PP100_DISCONNECTSERVER);
#endif//defined(_WIN32),defined(__GNUC__)
	if(	NULL == g_pPP100_Initialize ||
		NULL == g_pPP100_Destroy ||
		NULL == g_pPP100_ConnectServer ||
		NULL == g_pPP100_DisConnectServer)
	{
		printf("Getting function address failed.\n");
		return 1;
	}

	return 0;
}
