//=============================================================================
//	Step4
//
//	FileName	:	PP100APISampleStep4.cpp
//	Contents	:	Get PP-100 Series status and JOB status PP-100 Series API sample application.
//	Language	:	C++
//	Copyright	:	Copyright(C) SEIKO EPSON CORPORATION 2010. All rights reserved.
//	Note		:
//	 1.Outline
//	  Initialize, connect to PC, enumerate PP-100 Series, disconnect from PC, destroy, and output it return value to the console.
//	 2.Process
//	 (1)Initialize PP-100 Series API.
//		PP100_Initialize()
//	 (2)Connect to PC that TDBridge is running.
//		PP100_ConnectServer()
//	 (3)Enumerate PP-100 Series.
//		PP100_EnumPublishers()
//	 (4)Get PP-100 Series status.
//		PP100_GetPublisherStatus()
//	 (5)Get PP-100 Series JOB status.
//		PP100_GetJobStatus()
//	 (6)Disconnect from PC that TDBridge is running.
//		PP100_DisConnectServer()
//	 (7)Release PP-100 Series API.
//		PP100_Destroy()
//	CommandLine	:	PP100APISampleStep4
//=============================================================================

//=============================================================================
//	include files (#include)
//=============================================================================
#if defined(_WIN32)
#include <windows.h>
#include <tchar.h>
#include <shfolder.h>
#elif defined(__GNUC__)
#include <dlfcn.h>
#include <string.h>
#if !defined(_countof)
	template <typename Type, size_t Size>
	char (*__countof_helper(Type (&ar)[Size]))[Size];
	#define _countof(ar) sizeof(*__countof_helper(ar))
#endif//!defined(_countof)
#endif//defined(_WIN32),defined(__GNUC__)
#include <stdio.h>
#include "PP100API.h"

//=============================================================================
//	global variables
//=============================================================================
// The full path of PP100API.dll.
// If you changed the installation folder of EPSON TD Bridge, you need to changed the value.
#if defined(_WIN32)
TCHAR g_pPP100APIDLL[] = _T("\\EPSON\\TDBridge\\API\\PP100API.dll");
#elif defined(__GNUC__)
const char g_pPP100APIDLL[] = "/opt/epson/Total Disc Maker/lib/libPP100API.so";
#endif//defined(_WIN32),defined(__GNUC__)
HMODULE g_hPP100APIDLL = NULL;
PP100_INITIALIZE g_pPP100_Initialize = NULL;
PP100_DESTROY g_pPP100_Destroy = NULL;
PP100_CONNECTSERVER g_pPP100_ConnectServer = NULL;
PP100_DISCONNECTSERVER g_pPP100_DisConnectServer = NULL;
PP100_ENUMPUBLISHERS g_pPP100_EnumPublishers = NULL;
PP100_GETPUBLISHERSTATUS g_pPP100_GetPublisherStatus = NULL;
PP100_GETJOBSTATUS g_pPP100_GetJobStatus = NULL;

//=============================================================================
//	global functions
//=============================================================================
long LoadPP100API(); // Load PP-100 Series API and get function pointers.
void DisplayEnumPublisherInfo(PP100_ENUM_PUBLISHER_INFO pEnumPublisherInfo[], unsigned long ulEnumPublisherInfoNum); // Display PP100_ENUM_PUBLISHER_INFO.
void DisplayPublisherStatus(PP100_PUBLISHER_STATUS publisherStatus); // Display PP100_PUBLISHER_STATUS.
void DisplayJobStatus(PP100_JOB_STATUS pJobStatus[], unsigned long ulJobStatusNum);	// Display PP100_JOB_STATUS.

//=============================================================================
//	main
//
//	(1)Initialize PP-100 Series API.
//		PP100_Initialize()
//	(2)Connect to PC that TDBridge is running.
//		PP100_ConnectServer()
//	(3)Enumerate PP-100 Series.
//		PP100_EnumPublishers()
//	(4)Get PP-100 Series status.
//		PP100_GetPublisherStatus()
//	(5)Get PP-100 Series JOB status.
//		PP100_GetJobStatus()
//	(6)Disconnect from PC that TDBridge is running.
//		PP100_DisConnectServer()
//	(7)Release PP-100 Series API.
//		PP100_Destroy()
//
//	IN:
//		argc		- Number of input parameter.
//		argv		- Pointer to input paraemter.
//	OUT:
//
//	Return: void
//=============================================================================
int main(int argc, char *argv[])
{
#if defined(_WIN32)
	wchar_t pPublisherName[65] = {0};
#elif defined(__GNUC__)
	char pPublisherName[65] = {0};
#endif//defined(_WIN32),defined(__GNUC__)

	long nRet = LoadPP100API();
	printf("LoadPP100API() returns %ld\n", nRet);
	if(0 != nRet)
	{
		printf("Loading PP-100 Series API failed.\n");
		return nRet;
	}

	//=========================================================================
	// (1)Initialize PP100 Series API.
	//=========================================================================
	nRet = g_pPP100_Initialize();
	printf("PP100_Initialize() returns %ld\n", nRet);
	if(PP100API_SUCCESS != nRet)
	{
		printf("Initializing PP-100 Series API failed.\n");
		return nRet;
	}

	//=========================================================================
	// (2)Connect to PC that TDBridge is running.
	//    This sample sets pHost: 127.0.0.1, pOrderFolder: Orders
	//    i.e. \\127.0.0.1\Orders is full path of monitoring folder.
	//    If pHost is NULL, PP-100 Series API assume that local PC specified, and ignore pOrderFolder parameter.
	//=========================================================================
	unsigned long ulHandle = 0;
#if defined(_WIN32)
	nRet = g_pPP100_ConnectServer(L"127.0.0.1", L"Orders", &ulHandle);
#elif defined(__GNUC__)
	nRet = g_pPP100_ConnectServer(NULL, NULL, &ulHandle);
#endif//defined(_WIN32),defined(__GNUC__)
	printf("PP100_ConnectServer() returns %ld\n", nRet);
	printf("ulHandle is %lu\n", ulHandle);
	if(PP100API_SUCCESS != nRet)
	{
		printf("Connecting to PC that TDBridge is running failed.\n");
		return nRet;
	}

	//=========================================================================
	// (3)Enumerate PP-100 Series.
	//    First, call PP100_EnumPublishers() to get number of PP-100 Series.
	//    If return value is PP100API_MORE_ITEMS, PP100_EnumPublishers() succeeded.
	//    Then allocate new memory and call PP100_EnumPublishers() again.
	//=========================================================================
	PP100_ENUM_PUBLISHER_INFO* pEnumPublisherInfo = NULL;
	unsigned long ulEnumPublisherInfoNum = 0;
	nRet = g_pPP100_EnumPublishers(ulHandle, pEnumPublisherInfo, &ulEnumPublisherInfoNum);
	printf("PP100_EnumPublishers() returns %ld\n", nRet);
	printf("ulEnumPublisherInfoNum is %lu\n", ulEnumPublisherInfoNum);
	if(PP100API_MORE_ITEMS == nRet)
	{
		while(PP100API_MORE_ITEMS == nRet)
		{
			pEnumPublisherInfo = new PP100_ENUM_PUBLISHER_INFO[ulEnumPublisherInfoNum];
			if(NULL == pEnumPublisherInfo)
			{
				printf("Allocating new memory failed.\n");
				break;
			}
			else
			{
				memset(pEnumPublisherInfo, 0, sizeof(PP100_ENUM_PUBLISHER_INFO) * ulEnumPublisherInfoNum);
				nRet = g_pPP100_EnumPublishers(ulHandle, pEnumPublisherInfo, &ulEnumPublisherInfoNum);
				printf("PP100_EnumPublishers() returns %ld\n", nRet);
				if(PP100API_SUCCESS == nRet)
				{
					DisplayEnumPublisherInfo(pEnumPublisherInfo, ulEnumPublisherInfoNum);
				}
#if defined(_WIN32)
				wcscpy_s(pPublisherName, _countof(pPublisherName), pEnumPublisherInfo[0].pPublisherName);
#elif defined(__GNUC__)
				strcpy(pPublisherName, pEnumPublisherInfo[0].pPublisherName);
#endif//defined(_WIN32),defined(__GNUC__)
				delete[] pEnumPublisherInfo;
			}
		}
	}
	else
	{
		printf("Enumerating to PC that TDBridge is running failed.\n");
		return nRet;
	}

	//=========================================================================
	// (4)Get PP-100 Series status.
	//=========================================================================
#if defined(_WIN32)
	if(0 != wcslen(pPublisherName))
#elif defined(__GNUC__)
	if(0 != strlen(pPublisherName))
#endif//defined(_WIN32),defined(__GNUC__)
	{
		PP100_PUBLISHER_STATUS publisherStatus = {0};
		nRet = g_pPP100_GetPublisherStatus(ulHandle, pPublisherName, &publisherStatus);
		printf("PP100_GetPublisherStatus() returns %ld\n", nRet);
		if(PP100API_SUCCESS == nRet)
		{
			DisplayPublisherStatus(publisherStatus);
		}
		else
		{
			printf("Getting publisher status failed.\n");
		}
	}

	//=========================================================================
	// (5)Get PP-100 Series JOB Status.
	//    First, call PP100_GetJobStatus() to get number of PP-100 Series JOB Status.
	//    If return value is PP100API_MORE_ITEMS, PP100_GetJobStatus() succeeded.
	//    Then allocate new memory and call PP100_GetJobStatus() again.
	//=========================================================================
	PP100_JOB_STATUS* pJobStatus = NULL;
	unsigned long ulJobStatusNum = 0;
	nRet = g_pPP100_GetJobStatus(ulHandle, NULL, pJobStatus, &ulJobStatusNum);
	printf("PP100_GetJobStatus() returns %ld\n", nRet);
	printf("ulJobStatusNum is %lu\n", ulJobStatusNum);
	if(PP100API_MORE_ITEMS == nRet)
	{
		while(PP100API_MORE_ITEMS == nRet)
		{
			pJobStatus = new PP100_JOB_STATUS[ulJobStatusNum];
			if(NULL == pJobStatus)
			{
				printf("Allocating new memory failed.\n");
				break;
			}
			else
			{
				memset(pJobStatus, 0, sizeof(PP100_JOB_STATUS) * ulJobStatusNum);
				nRet = g_pPP100_GetJobStatus(ulHandle, NULL, pJobStatus, &ulJobStatusNum);
				printf("PP100_GetJobStatus() returns %ld\n", nRet);
				if(PP100API_SUCCESS == nRet)
				{
					DisplayJobStatus(pJobStatus, ulJobStatusNum);
				}
				delete[] pJobStatus;
			}
		}
	}
	else
	{
		printf("Getting PP-100 Series JOB Status failed.\n");
		return nRet;
	}

	//=========================================================================
	// (6)Disconnecting from PC that TDBridge is running.
	//=========================================================================
	nRet = g_pPP100_DisConnectServer(ulHandle);
	printf("PP100_DisConnectServer() returns %ld\n", nRet);
	if(PP100API_SUCCESS != nRet)
	{
		printf("Disconnecting from PC that TDBridge is running failed.\n");
		return nRet;
	}

	//=========================================================================
	// (7)Release PP100 Series API.
	//=========================================================================
	nRet = g_pPP100_Destroy();
	printf("PP100_Destroy() returns %ld\n", nRet);
	if(PP100API_SUCCESS != nRet)
	{
		printf("Destroying PP-100 Series API failed.\n");
		return nRet;
	}

	return nRet;
}

//=============================================================================
//	LoadPP100API
//
//	(1)Load PP-100 Series API and get function pointers.
//
//	IN:
//	OUT:
//
//	Return:	0	- success
//			1	- failure
//=============================================================================
long LoadPP100API()
{
	// Load PP100API.dll

#if defined(_WIN32)
	TCHAR tcDLLPath[MAX_PATH];
	HRESULT hResult;
	hResult = SHGetFolderPath(NULL, CSIDL_PROGRAM_FILES, NULL, 0, tcDLLPath);
	if(S_OK != hResult)
	{
		printf("Loading PP-100 Series API failed.\n");
		return 1;
	}

	_tcscat_s(tcDLLPath, _countof(tcDLLPath), g_pPP100APIDLL);

	g_hPP100APIDLL = ::LoadLibrary(tcDLLPath);
	if(NULL == g_hPP100APIDLL)
	{
		printf("Loading PP-100 Series API failed.\n");
		return 1;
	}

	// Get function pointer of PP100_Initialize() and PP100_Destroy()
	g_pPP100_Initialize = (PP100_INITIALIZE)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_INITIALIZE);
	g_pPP100_Destroy = (PP100_DESTROY)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_DESTROY);
	g_pPP100_ConnectServer = (PP100_CONNECTSERVER)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_CONNECTSERVER);
	g_pPP100_DisConnectServer = (PP100_DISCONNECTSERVER)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_DISCONNECTSERVER);
	g_pPP100_EnumPublishers = (PP100_ENUMPUBLISHERS)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_ENUMPUBLISHERS);
	g_pPP100_GetPublisherStatus = (PP100_GETPUBLISHERSTATUS)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_GETPUBLISHERSTATUS);
	g_pPP100_GetJobStatus = (PP100_GETJOBSTATUS)GetProcAddress(g_hPP100APIDLL, FNSTR_PP100_GETJOBSTATUS);
#elif defined(__GNUC__)
	const char *the_path = g_pPP100APIDLL;
	g_hPP100APIDLL = dlopen(the_path, RTLD_LAZY);
	if(NULL == g_hPP100APIDLL)
	{
		printf("Loading PP-100 Series API failed.\n");
		return 1;
	}

	// Get function pointer of PP100_Initialize() and PP100_Destroy()
	g_pPP100_Initialize = (PP100_INITIALIZE)dlsym(g_hPP100APIDLL, FNSTR_PP100_INITIALIZE);
	g_pPP100_Destroy = (PP100_DESTROY)dlsym(g_hPP100APIDLL, FNSTR_PP100_DESTROY);
	g_pPP100_ConnectServer = (PP100_CONNECTSERVER)dlsym(g_hPP100APIDLL, FNSTR_PP100_CONNECTSERVER);
	g_pPP100_DisConnectServer = (PP100_DISCONNECTSERVER)dlsym(g_hPP100APIDLL, FNSTR_PP100_DISCONNECTSERVER);
	g_pPP100_EnumPublishers = (PP100_ENUMPUBLISHERS)dlsym(g_hPP100APIDLL, FNSTR_PP100_ENUMPUBLISHERS);
	g_pPP100_GetPublisherStatus = (PP100_GETPUBLISHERSTATUS)dlsym(g_hPP100APIDLL, FNSTR_PP100_GETPUBLISHERSTATUS);
	g_pPP100_GetJobStatus = (PP100_GETJOBSTATUS)dlsym(g_hPP100APIDLL, FNSTR_PP100_GETJOBSTATUS);
#endif//defined(_WIN32),defined(__GNUC__)

	if(	NULL == g_pPP100_Initialize ||
		NULL == g_pPP100_Destroy ||
		NULL == g_pPP100_ConnectServer ||
		NULL == g_pPP100_DisConnectServer ||
		NULL == g_pPP100_EnumPublishers ||
		NULL == g_pPP100_GetPublisherStatus ||
		NULL == g_pPP100_GetJobStatus)
	{
		printf("Getting function address failed.\n");
		return 1;
	}

	return 0;
}

//=============================================================================
//	DisplayEnumPublisherInfo
//
//	(1)Display PP100_ENUM_PUBLISHER_INFO.
//
//	IN:
//		pEnumPublisherInfo		- Pointer to PP100_ENUM_PUBLISHER_INFO array.
//		ulEnumPublisherInfoNum	- Number of array.
//	OUT:
//
//	Return: void
//=============================================================================
void DisplayEnumPublisherInfo(PP100_ENUM_PUBLISHER_INFO pEnumPublisherInfo[], unsigned long ulEnumPublisherInfoNum)
{
	for(unsigned long ul = 0; ul < ulEnumPublisherInfoNum; ul++)
	{
#if defined(_WIN32)
		wprintf(L"\tpEnumPublisherInfo[%lu].pPublisherName is %s\n", ul, pEnumPublisherInfo[ul].pPublisherName);
		wprintf(L"\tpEnumPublisherInfo[%lu].ulDriveNumber is %lu\n", ul, pEnumPublisherInfo[ul].ulDriveNumber);
		wprintf(L"\tpEnumPublisherInfo[%lu].ulConnectType is %lu\n", ul, pEnumPublisherInfo[ul].ulConnectType);
#elif defined(__GNUC__)
		printf("\tpEnumPublisherInfo[%lu].pPublisherName is %s\n", ul, pEnumPublisherInfo[ul].pPublisherName);
		printf("\tpEnumPublisherInfo[%lu].ulDriveNumber is %lu\n", ul, pEnumPublisherInfo[ul].ulDriveNumber);
		printf("\tpEnumPublisherInfo[%lu].ulConnectType is %lu\n", ul, pEnumPublisherInfo[ul].ulConnectType);
#endif//defined(_WIN32),defined(__GNUC__)
	}
}

//=============================================================================
//	DisplayPublisherStatus
//
//	(1)Display PP100_PUBLISHER_STATUS.
//
//	IN:
//		publisherStatus			- PP100_PUBLISHER_STATUS structure.
//	OUT:
//
//	Return: void
//=============================================================================
void DisplayPublisherStatus(PP100_PUBLISHER_STATUS publisherStatus)
{
#if defined(_WIN32)
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulINFORMATIONCode); ul++)
	{
		if(0 == publisherStatus.ulINFORMATIONCode[ul])
		{
			break;
		}
		wprintf(L"\tpublisherStatus.ulINFORMATIONCode[%lu]: %lu\n", ul, publisherStatus.ulINFORMATIONCode[ul]);
	}
	wprintf(L"\tstPublisherStatus.ulMode: %lu\n", publisherStatus.ulMode);
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulDriveStatus); ul++)
	{
		wprintf(L"\tpublisherStatus.ulDriveStatus[%lu]: %lu\n", ul, publisherStatus.ulDriveStatus[ul]);
	}
	for(unsigned long ul = 0; ul < _countof(publisherStatus.pDrivePluginName); ul++)
	{
		wprintf(L"\tpublisherStatus.pDrivePluginName[%lu]: %s\n", ul, publisherStatus.pDrivePluginName[ul]);
	}
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulDriveLife); ul++)
	{
		wprintf(L"\tpublisherStatus.ulDriveLife[%lu]: %lu\n", ul, publisherStatus.ulDriveLife[ul]);
	}
	wprintf(L"\tpublisherStatus.ulPrinterStatus: %lu\n", publisherStatus.ulPrinterStatus);
	wprintf(L"\tpublisherStatus.stInkStatus.ulCyan: %lu\n", publisherStatus.stInkStatus.ulCyan);
	wprintf(L"\tpublisherStatus.stInkStatus.ulMagenta: %lu\n", publisherStatus.stInkStatus.ulMagenta);
	wprintf(L"\tpublisherStatus.stInkStatus.ulYellow: %lu\n", publisherStatus.stInkStatus.ulYellow);
	wprintf(L"\tpublisherStatus.stInkStatus.ulLightCyan: %lu\n", publisherStatus.stInkStatus.ulLightCyan);
	wprintf(L"\tpublisherStatus.stInkStatus.ulLightMagenta: %lu\n", publisherStatus.stInkStatus.ulLightMagenta);
	wprintf(L"\tpublisherStatus.stInkStatus.ulBlack: %lu\n", publisherStatus.stInkStatus.ulBlack);
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulStackerSetting); ul++)
	{
		wprintf(L"\tpublisherStatus.ulStackerSetting[%lu]: %lu\n", ul, publisherStatus.ulStackerSetting[ul]);
	}
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulStackerRest); ul++)
	{
		wprintf(L"\tpublisherStatus.ulStackerRest[%lu]: %lu\n", ul, publisherStatus.ulStackerRest[ul]);
	}
	wprintf(L"\tpublisherStatus.ulPrintableCopies: %lu\n", publisherStatus.ulPrintableCopies);
	wprintf(L"\tpublisherStatus.ulPrintedCopies: %lu\n", publisherStatus.ulPrintedCopies);
	wprintf(L"\tpublisherStatus.ulMaintenanceBoxFreeSpace: %lu\n", publisherStatus.ulMaintenanceBoxFreeSpace);
	wprintf(L"\tpublisherStatus.pSerialNumber: %s\n", publisherStatus.pSerialNumber);
#elif defined(__GNUC__)
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulINFORMATIONCode); ul++)
	{
		if(0 == publisherStatus.ulINFORMATIONCode[ul])
		{
			break;
		}
		printf("\tpublisherStatus.ulINFORMATIONCode[%lu]: %lu\n", ul, publisherStatus.ulINFORMATIONCode[ul]);
	}
	printf("\tstPublisherStatus.ulMode: %lu\n", publisherStatus.ulMode);
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulDriveStatus); ul++)
	{
		printf("\tpublisherStatus.ulDriveStatus[%lu]: %lu\n", ul, publisherStatus.ulDriveStatus[ul]);
	}
	for(unsigned long ul = 0; ul < _countof(publisherStatus.pDrivePluginName); ul++)
	{
		printf("\tpublisherStatus.pDrivePluginName[%lu]: %s\n", ul, publisherStatus.pDrivePluginName[ul]);
	}
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulDriveLife); ul++)
	{
		printf("publisherStatus.ulDriveLife[%lu]: %lu\n", ul, publisherStatus.ulDriveLife[ul]);
	}
	printf("\tpublisherStatus.ulPrinterStatus: %lu\n", publisherStatus.ulPrinterStatus);
	printf("\tpublisherStatus.stInkStatus.ulCyan: %lu\n", publisherStatus.stInkStatus.ulCyan);
	printf("\tpublisherStatus.stInkStatus.ulMagenta: %lu\n", publisherStatus.stInkStatus.ulMagenta);
	printf("\tpublisherStatus.stInkStatus.ulYellow: %lu\n", publisherStatus.stInkStatus.ulYellow);
	printf("\tpublisherStatus.stInkStatus.ulLightCyan: %lu\n", publisherStatus.stInkStatus.ulLightCyan);
	printf("\tpublisherStatus.stInkStatus.ulLightMagenta: %lu\n", publisherStatus.stInkStatus.ulLightMagenta);
	printf("\tpublisherStatus.stInkStatus.ulBlack: %lu\n", publisherStatus.stInkStatus.ulBlack);
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulStackerSetting); ul++)
	{
		printf("\tpublisherStatus.ulStackerSetting[%lu]: %lu\n", ul, publisherStatus.ulStackerSetting[ul]);
	}
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulStackerRest); ul++)
	{
		printf("\tpublisherStatus.ulStackerRest[%lu]: %lu\n", ul, publisherStatus.ulStackerRest[ul]);
	}
	printf("\tpublisherStatus.ulPrintableCopies: %lu\n", publisherStatus.ulPrintableCopies);
	printf("\tpublisherStatus.ulPrintedCopies: %lu\n", publisherStatus.ulPrintedCopies);
	printf("\tpublisherStatus.ulMaintenanceBoxFreeSpace: %lu\n", publisherStatus.ulMaintenanceBoxFreeSpace);
	printf("\tpublisherStatus.pSerialNumber: %s\n", publisherStatus.pSerialNumber);
#endif//defined(_WIN32),defined(__GNUC__)
}

//=============================================================================
//	DisplayJobStatus
//
//	(1)Display PP100_JOB_STATUS.
//
//	IN:
//		pJobStatus		- Pointer to PP100_JOB_STATUS array.
//		ulJobStatusNum	- Number of array.
//	OUT:
//
//	Return: void
//=============================================================================
void DisplayJobStatus(PP100_JOB_STATUS pJobStatus[], unsigned long ulJobStatusNum)
{
	for(unsigned long ul = 0; ul < ulJobStatusNum; ul++)
	{
#if defined(_WIN32)
		wprintf(L"\tpJobStatus[%lu].pJobID: %s\n", ul, pJobStatus[ul].pJobID);
		wprintf(L"\tpJobStatus[%lu].pPublisherName: %s\n", ul, pJobStatus[ul].pPublisherName);
		wprintf(L"\tpJobStatus[%lu].ulJobStatus: %lu\n", ul, pJobStatus[ul].ulJobStatus);
		wprintf(L"\tpJobStatus[%lu].ulErrorCode: %lu\n", ul, pJobStatus[ul].ulErrorCode);
		wprintf(L"\tpJobStatus[%lu].ulPublicationNumber: %lu\n", ul, pJobStatus[ul].ulPublicationNumber);
		wprintf(L"\tpJobStatus[%lu].ulCompletionNumber: %lu\n", ul, pJobStatus[ul].ulCompletionNumber);
		wprintf(L"\tpJobStatus[%lu].ulJobType: %lu\n", ul, pJobStatus[ul].ulJobType);
		wprintf(L"\tpJobStatus[%lu].ulSource: %lu\n", ul, pJobStatus[ul].ulSource);
		wprintf(L"\tpJobStatus[%lu].ulDestination: %lu\n", ul, pJobStatus[ul].ulDestination);
		wprintf(L"\tpJobStatus[%lu].ulJobIndex: %lu\n", ul, pJobStatus[ul].ulJobIndex);
		wprintf(L"\tpJobStatus[%lu].ulEstimateTime: %lu\n", ul, pJobStatus[ul].ulEstimateTime);
		wprintf(L"\tpJobStatus[%lu].ulErrorNumber: %lu\n", ul, pJobStatus[ul].ulErrorNumber);
		wprintf(L"\tpJobStatus[%lu].ulMode: %lu\n", ul, pJobStatus[ul].ulMode);
		wprintf(L"\tpJobStatus[%lu].ucIsSubmittedByTotalDiscMaker: %hhu\n", ul, pJobStatus[ul].ucIsSubmittedByTotalDiscMaker);
#elif defined(__GNUC__)
		printf("\tpJobStatus[%lu].pJobID: %s\n", ul, pJobStatus[ul].pJobID);
		printf("\tpJobStatus[%lu].pPublisherName: %s\n", ul, pJobStatus[ul].pPublisherName);
		printf("\tpJobStatus[%lu].ulJobStatus: %lu\n", ul, pJobStatus[ul].ulJobStatus);
		printf("\tpJobStatus[%lu].ulErrorCode: %lu\n", ul, pJobStatus[ul].ulErrorCode);
		printf("\tpJobStatus[%lu].ulPublicationNumber: %lu\n", ul, pJobStatus[ul].ulPublicationNumber);
		printf("\tpJobStatus[%lu].ulCompletionNumber: %lu\n", ul, pJobStatus[ul].ulCompletionNumber);
		printf("\tpJobStatus[%lu].ulJobType: %lu\n", ul, pJobStatus[ul].ulJobType);
		printf("\tpJobStatus[%lu].ulSource: %lu\n", ul, pJobStatus[ul].ulSource);
		printf("\tpJobStatus[%lu].ulDestination: %lu\n", ul, pJobStatus[ul].ulDestination);
		printf("\tpJobStatus[%lu].ulJobIndex: %lu\n", ul, pJobStatus[ul].ulJobIndex);
		printf("\tpJobStatus[%lu].ulEstimateTime: %lu\n", ul, pJobStatus[ul].ulEstimateTime);
		printf("\tpJobStatus[%lu].ulErrorNumber: %lu\n", ul, pJobStatus[ul].ulErrorNumber);
		printf("\tpJobStatus[%lu].ulMode: %lu\n", ul, pJobStatus[ul].ulMode);
		printf("\tpJobStatus[%lu].ucIsSubmittedByTotalDiscMaker: %hhu\n", ul, pJobStatus[ul].ucIsSubmittedByTotalDiscMaker);
#endif//defined(_WIN32),defined(__GNUC__)
	}
}
