#include <string>
#include <assert.h>
#include <QtCore/QString>
#include <QDialog>
#include <QtGui/QWindowStateChangeEvent>

#include "TDMonitor.h"
#include "ui_VersionDialog.h"
#include "TDMonitor.moc"

QString gDriveWriteStyle = "QProgressBar { border: 1px solid grey; border-radius: 5px; text-align: center;} QProgressBar::chunk {border-top: 0.5px solid #000; border-radius: 3px; margin: 0.5px; background-color: #55AAFF;}";
QString gDriveCompareStyle = "QProgressBar { border: 1px solid grey; border-radius: 5px; text-align: center;} QProgressBar::chunk {border-top: 0.5px solid #000; border-radius: 3px; margin: 0.5px; background-color: #00E000; text-align: center;}";

// PP100API End processing
long endPP100Api()
{
	// Disconnecting from PC that TDBridge is running.
	long nRet = g_Api.PP100_DisConnectServer(g_Api.ulHandle);
	if(PP100API_SUCCESS != nRet)
	{
		printf("Disconnecting from PC that TDBridge is running failed.\n");
		return nRet;
	}

	// Release PP100 Series API.
	nRet = g_Api.PP100_Destroy();
	if(PP100API_SUCCESS != nRet)
	{
		printf("Destroying PP-100 Series API failed.\n");
	}
	return nRet;
}


TDMonitorEx::TDMonitorEx(const char szPublisherName[], QMainWindow *parent)
{
	assert(szPublisherName);

	strcpy(m_cPublisherName, szPublisherName);
	setupUi(this);
	// Screen update cycle setting (normal time: 5000msec, window minimization: stop)
	m_timer.reset(new QTimer(this));
	m_timer->setInterval(5000);
	m_timer->setSingleShot(false);

	// Itegrate 2 actions into one by using QActionGroup
	m_exclusivActionShowJobs.reset(new QActionGroup(this));
	m_exclusivActionShowJobs->setExclusive(true);
	actionShowUnfinishedJobs->setActionGroup(m_exclusivActionShowJobs.get());
	actionShowFinishedJobs->setActionGroup(m_exclusivActionShowJobs.get());

	// TreeWidget settings
	treeJobList->setSelectionMode(QAbstractItemView::MultiSelection);
	treeJobList->setColumnWidth(0, 240);
	QTreeWidgetItem *header = treeJobList->headerItem();
	header->setFlags(header->flags() ^ Qt::ItemIsDragEnabled);
	treeJobList->clear();

	// SIGNAL,SLOT settings
	QObject::connect(actionCancelJob, SIGNAL(triggered()), this, SLOT(cancelJob()));
	QObject::connect(actionSelectAllJobs, SIGNAL(triggered()), this, SLOT(selectAllJobs()));
	QObject::connect(actionShowUnfinishedJobs, SIGNAL(triggered()), this, SLOT(showUnfinishedJobs()));
	QObject::connect(actionShowFinishedJobs, SIGNAL(triggered()), this, SLOT(showFinishedJobs()));
	QObject::connect(actionShowAbout, SIGNAL(triggered()), this, SLOT(showAboutDialog()));
	QObject::connect(tabWidget, SIGNAL(currentChanged(int)), this, SLOT(changeTab(int)));
	QObject::connect(m_timer.get(), SIGNAL(timeout()), this, SLOT(updatePeriodical()));
	QObject::connect(treeJobList, SIGNAL(clicked(QModelIndex)), this, SLOT(updateMenu()));

	// Set context menu for TreeWidget
	treeJobList->addAction(actionCancelJob);
	treeJobList->addAction(actionSelectAllJobs);

	// Add Publisher name to window title
	QString title = windowTitle() + m_cPublisherName;
	setWindowTitle(title);

	adjustSize();

	updateContent();
	updateMenu();
	m_timer->start();
}

TDMonitorEx::~TDMonitorEx()
{
	endPP100Api();
}

// Screen information update
void TDMonitorEx::updateContent(void)
{
	std::deque<PP100_JOB_STATUS> jobStatusPublishing;
	std::deque<PP100_JOB_STATUS> jobStatusFinished;
	unsigned long ulJobStatusNum = 0;
	long nRet = g_Api.PP100_GetJobStatus(g_Api.ulHandle, NULL, NULL, &ulJobStatusNum);
	// Job list creation
	if(PP100API_MORE_ITEMS == nRet)
	{
		while(PP100API_MORE_ITEMS == nRet)
		{
			std::vector<PP100_JOB_STATUS> jobStatus(ulJobStatusNum);
			nRet = g_Api.PP100_GetJobStatus(g_Api.ulHandle, NULL, &jobStatus[0], &ulJobStatusNum);
			if(PP100API_SUCCESS == nRet)
			{
				for(unsigned long ulCnt = 0; ulCnt < ulJobStatusNum; ++ulCnt)
				{
					if(strcmp(jobStatus[ulCnt].pPublisherName, m_cPublisherName) == 0)
					{
						if (
							jobStatus[ulCnt].ulJobStatus != PP100API_JOB_PUBLISHED &&
							jobStatus[ulCnt].ulJobStatus != PP100API_JOB_USER_CANCELLED &&
							jobStatus[ulCnt].ulJobStatus != PP100API_JOB_ERROR_CANCELLED &&
							jobStatus[ulCnt].ulJobStatus != PP100API_JOB_PUBLISHED_WARNING_NO_ERROR_DISC &&
							jobStatus[ulCnt].ulJobStatus != PP100API_JOB_PUBLISHED_WARNING_ERROR_DISC )
						{
							jobStatusPublishing.push_back(jobStatus[ulCnt]);
						}
						else
						{
							jobStatusFinished.push_front(jobStatus[ulCnt]);
						}
					}
				}
			}
		}
	}
	else if (PP100API_SUCCESS != nRet)
	{
		printf("Getting PP-100 Series JOB Status failed.\n");
	}

	treeJobList->clear();
	if(tabWidget->currentIndex() == 0)
	{
		for(PP100_JOB_STATUS & jobStatus : jobStatusPublishing)
		{
			appendTreeItem(jobStatus);
		}
	}
	else
	{
		for(PP100_JOB_STATUS & jobStatus : jobStatusFinished)
		{
			appendTreeItem(jobStatus);
		}
	}

	// Device Status
	PP100_PUBLISHER_STATUS publisherStatus = {0};
	nRet = g_Api.PP100_GetPublisherStatus(g_Api.ulHandle, m_cPublisherName, &publisherStatus);
	if(PP100API_SUCCESS == nRet)
	{
		// set text on textEdit_DeviceStatus based on the status
		setDeviceInfo(publisherStatus, jobStatusPublishing);
	}
	else
	{
		textEdit_DeviceStatus->setText(
			tr("L7007:\n"
				"Communication with the specified output device has been interrupted.\n"
				"The communication settings may have been changed.\n"
				"Restart Total Disc Monitor."));
	}

	// Ink Status
	barInkC->setValue(publisherStatus.stInkStatus.ulCyan);
	barInkLC->setValue(publisherStatus.stInkStatus.ulLightCyan);
	barInkLM->setValue(publisherStatus.stInkStatus.ulLightMagenta);
	barInkM->setValue(publisherStatus.stInkStatus.ulMagenta);
	barInkY->setValue(publisherStatus.stInkStatus.ulYellow);
	barInkK->setValue(publisherStatus.stInkStatus.ulBlack);

	// Drive Status
	setDriveInfo(publisherStatus, 0);
	setDriveInfo(publisherStatus, 1);

	// Printer Status
	switch(publisherStatus.ulPrinterStatus)
	{
	case PP100API_TRANSFERRING:
		label_PrinterStatus->setText(tr("Transferring"));
		break;
	case PP100API_PRINTING:
		label_PrinterStatus->setText(tr("Printing"));
		break;
	case PP100API_PAUSED:
		label_PrinterStatus->setText(tr("Stopping"));
		break;
	case PP100API_CLEANING:
		label_PrinterStatus->setText(tr("Cleaning"));
		break;
	case PP100API_DRYING:
		label_PrinterStatus->setText(tr("Drying Ink"));
		break;
	case PP100API_ERROR:
		label_PrinterStatus->setText(tr("Error"));
		break;
	case PP100API_WAITING:
		label_PrinterStatus->setText(tr("Standby"));
		break;
	default:
		fprintf(stderr, "unknown printer status: %lu\n", publisherStatus.ulPrinterStatus);
		label_PrinterStatus->setText(tr("Unknown"));
		break;
	}

	// Mode
	switch(publisherStatus.ulMode)
	{
	case PP100API_OUTPUT_MODE:
		label_Mode->setText(tr("External Output / Low-volume mode"));
		break;
	case PP100API_BATCH_MODE:
		label_Mode->setText(tr("Batch mode"));
		break;
	case PP100API_NORMAL_MODE:
		label_Mode->setText(tr("Standard mode"));
		break;
	default:
		fprintf(stderr, "unknown publish mode: %lu\n", publisherStatus.ulMode);
		label_Mode->setText(tr("Unknown"));
		break;
	}

	// Stacker status setting
	for(int nCnt = 0; nCnt < 4; ++nCnt)
	{
		setStackerSetting(publisherStatus, nCnt);
	}

	// Check if you are connected with USB3.0 and set the icon
	PP100_PUBLISHER_STATUS status;

	if(g_Api.PP100_GetPublisherStatus(g_Api.ulHandle, m_cPublisherName, &status) == PP100API_SUCCESS)
	{
		if(PP100API_USB_30 == status.ulUsbConnectionMode)
		{
			iconUSB->setPixmap(QPixmap(QString::fromUtf8(":/res/E27-1.png")));
		}
		else
		{
			iconUSB->setPixmap(QPixmap(QString::fromUtf8(":/res/E27-2.png")));
		}
	}
	else
	{
		iconUSB->setPixmap(QPixmap(QString::fromUtf8(":/res/E27-2.png")));
	}

	repaint();
}

// Add TreeWidgetItem with Job information
void TDMonitorEx::appendTreeItem(const PP100_JOB_STATUS& jobStatus)
{
	if (
		jobStatus.ulJobStatus != PP100API_JOB_PUBLISHED &&
		jobStatus.ulJobStatus != PP100API_JOB_USER_CANCELLED &&
		jobStatus.ulJobStatus != PP100API_JOB_ERROR_CANCELLED &&
		jobStatus.ulJobStatus != PP100API_JOB_PUBLISHED_WARNING_NO_ERROR_DISC &&
		jobStatus.ulJobStatus != PP100API_JOB_PUBLISHED_WARNING_ERROR_DISC )
	{
		// publishing job
		if (tabWidget->currentIndex() == 1)
		{
			// If the job does not meet the display conditions,
			// it ends (Completed JOB on the Waiting for Issuance tab or Waiting for Issuing on the Completed JOB tab)
			return;
		}
	}
	else
	{
		// finished job
		if (tabWidget->currentIndex() == 0)
		{
			// If the job does not meet the display conditions,
			// it ends (Completed JOB on the Waiting for Issuance tab or Waiting for Issuing on the Completed JOB tab)
			return;
		}
	}

	QTreeWidgetItem* pItem = new QTreeWidgetItem(treeJobList);

	if(!tabWidget->currentIndex())
	{
		// Unfinished Job
		pItem->setTextAlignment(2, Qt::AlignRight|Qt::AlignVCenter);
		pItem->setTextAlignment(3, Qt::AlignLeft|Qt::AlignVCenter);
		pItem->setTextAlignment(4, Qt::AlignLeft|Qt::AlignVCenter);
		// Time Needed
		if(jobStatus.ulRemainingEstimateTime > 3600)
		{
			pItem->setText(
				6,
				QString::asprintf("%lu ", (jobStatus.ulRemainingEstimateTime / 3600))
				+ tr("hr")
				+ QString::asprintf(" %lu ", ((jobStatus.ulRemainingEstimateTime % 3600) / 60))
				+ tr("min"));
		}
		else if(jobStatus.ulRemainingEstimateTime > 60)
		{
			pItem->setText(
				6,
				QString::asprintf("%lu ", (jobStatus.ulRemainingEstimateTime / 60))
				+ tr("min"));
		}
		else
		{
			pItem->setText(6, tr("1 min"));
		}
		// Output Stacker
		setStacker_out(5, jobStatus, pItem);
		// Input Stacker
		setStacker_in(4, jobStatus, pItem);
		// Type
		setJobType(3, jobStatus, pItem);
		// Copies
		pItem->setText(2, QString::asprintf("%lu / %lu", jobStatus.ulCompletionNumber, jobStatus.ulPublicationNumber));
	}
	else
	{
		// Finished Job
		pItem->setTextAlignment(2, Qt::AlignRight|Qt::AlignVCenter);
		pItem->setTextAlignment(3, Qt::AlignRight|Qt::AlignVCenter);
		pItem->setTextAlignment(4, Qt::AlignRight|Qt::AlignVCenter);

		// Output Stacker
		setStacker_out(6, jobStatus, pItem);
		// Type
		setJobType(5, jobStatus, pItem);
		// Quantity
		pItem->setText(4, QString::asprintf("%lu", jobStatus.ulPublicationNumber));
		// Failed
		pItem->setText(3, QString::asprintf("%lu", jobStatus.ulErrorNumber));
		// Successful
		pItem->setText(2, QString::asprintf("%lu", jobStatus.ulCompletionNumber));
	}
	// Status
	switch(jobStatus.ulJobStatus)
	{
	case PP100API_JOB_RUNNING:
		pItem->setText(1, tr("Publishing"));
		break;
	case PP100API_JOB_CANCELING:
		pItem->setText(1, tr("Canceling"));
		break;
	case PP100API_JOB_PAUSING:
		pItem->setText(1, tr("Pausing"));
		break;
	case PP100API_JOB_PAUSED:
		pItem->setText(1, tr("Paused"));
		break;
	case PP100API_JOB_USER_CANCELLED:
		pItem->setText(1, tr("Cancel"));
		break;
	case PP100API_JOB_PUBLISHED:
		pItem->setText(1, tr("Published"));
		break;
	case PP100API_JOB_WAITING:
		pItem->setText(1, tr("Waiting"));
		break;
	case PP100API_JOB_RECOVERING:
		pItem->setText(1, tr("Recovering"));
		break;
	case PP100API_JOB_STANDBY:
		pItem->setText(1, tr("Standby_job"));
		break;
	case PP100API_JOB_PUBLISHED_WARNING_NO_ERROR_DISC:
	case PP100API_JOB_PUBLISHED_WARNING_ERROR_DISC:
		pItem->setText(1, tr("Published (Warning)"));
		break;
	case PP100API_JOB_ERROR_CANCELLED:
		pItem->setText(1, tr("Error"));
		break;
	default:
		fprintf(stderr, "unknown job status: %lu\n", jobStatus.ulJobStatus);
		pItem->setText(1, tr("Unknown"));
		break;
	}
	// Job Name
	pItem->setText(0, jobStatus.pJobID);
}

// TreeWidget discharge source setting process
void TDMonitorEx::setStacker_in(int nCol, const PP100_JOB_STATUS& jobStatus, QTreeWidgetItem *pItem)
{
	switch(jobStatus.ulSource)
	{
	case PP100API_STACKER1:
		pItem->setText(nCol, "1");
		break;
	case PP100API_STACKER2:
		pItem->setText(nCol, "2");
		break;
	case PP100API_STACKER3:
		pItem->setText(nCol, "3");
		break;
	case PP100API_STACKER4:
		pItem->setText(nCol, "4");
		break;
	case PP100API_STACKER1_OR_STACKER2:
		if(PP100API_BATCH_MODE==jobStatus.ulMode)
		{
			pItem->setText(nCol, "2, 1");
		}
		else
		{
			pItem->setText(nCol, "1, 2");
		}
		break;
	case PP100API_STACKER2_OR_STACKER3:
		if(PP100API_BATCH_MODE==jobStatus.ulMode)
		{
			pItem->setText(nCol, "3, 2");
		}
		else
		{
			pItem->setText(nCol, "2, 3");
		}
		break;
	default:
		pItem->setText(1, tr("Unknown"));
		break;
	}
}

// TreeWidget discharge destination setting process
void TDMonitorEx::setStacker_out(int nCol, const PP100_JOB_STATUS& jobStatus, QTreeWidgetItem *pItem)
{
	switch(jobStatus.ulDestination)
	{
	case PP100API_STACKER1:
		pItem->setText(nCol, "1");
		break;
	case PP100API_STACKER2:
		pItem->setText(nCol, "2");
		break;
	case PP100API_STACKER3:
		pItem->setText(nCol, "3");
		break;
	case PP100API_STACKER4:
		pItem->setText(nCol, "4");
		break;
	case PP100API_STACKER2_OR_STACKER3:
		if(PP100API_BATCH_MODE==jobStatus.ulMode)
		{
			pItem->setText(nCol, "3, 2");
		}
		else
		{
			pItem->setText(nCol, "2, 3");
		}
		break;
	case PP100API_STACKER2_OR_STACKER4:
		pItem->setText(nCol, "2, 4");
		break;
	default:
		pItem->setText(1, tr("Unknown"));
		break;
	}
}

// TreeWidget type setting process
void TDMonitorEx::setJobType(int nCol, const PP100_JOB_STATUS& jobStatus, QTreeWidgetItem *pItem)
{
	switch(jobStatus.ulJobType)
	{
	case PP100API_JOB_TYPE_PUBLISH:
		pItem->setText(nCol, tr("Publish"));
		break;
	case PP100API_JOB_TYPE_WRITE:
		pItem->setText(nCol, tr("Write"));
		break;
	case PP100API_JOB_TYPE_PRINT:
		pItem->setText(nCol, tr("Print"));
		break;
	case PP100API_JOB_TYPE_MEASURE:
		pItem->setText(nCol, tr("Measure"));
		break;
	default:
		fprintf(stderr, "unknown job type: %lu\n", jobStatus.ulJobType);
	case PP100API_ERROR:
		pItem->setText(nCol, tr("Unknown"));
		break;
	}
}

// Device state settings
void TDMonitorEx::setDeviceInfo(const PP100_PUBLISHER_STATUS& publisherStatus, const std::deque<PP100_JOB_STATUS>& jobStatusPublishing)
{
	QString msg;

	textEdit_DeviceStatus->clear();
	for(unsigned long ul = 0; ul < _countof(publisherStatus.ulINFORMATIONCode) && publisherStatus.ulINFORMATIONCode[ul] != 0; ul++)
	{
		if(ul)
		{
			msg += "\n";
		}
		switch(publisherStatus.ulINFORMATIONCode[ul])
		{
		case PP100API_HPR000:
			// A printer fatal error has occurred.
			msg += tr("L7011:\n"
				"An internal printer error occurred.\n"
				"Turn off the output device, check inside the printer, and then turn on the output device again.\n"
				"If the same error occurs frequently, contact Customer Support.");
			break;

		case PP100API_HPR001:
		case PP100API_HPR006:
			// A printer maintenance error has occurred.
			msg += tr("L7010:\n"
				"A printer maintenance error occurred.\n"
				"For details, please contact support.");
			break;

		case PP100API_HPR002:
			// The waste ink pads need to be replaced.
			msg += tr("L7009:\n"
				"Ink Pad Full\n"
				"\n"
				"The ink pad for this device is full and the device can no longer function.\n"
				"Please consult your manual for the recommended action.");
			break;

		case PP100API_HPR003:
			// A communication with the printer is failed.
			msg += tr("L7016:\n"
				"Communication with the printer in the output device has been interrupted.\n"
				"The port settings for the printer driver may have been changed.\n"
				"Correct the port settings.");
			break;

		case PP100API_HPR005:
			// The ink cartridge cover is open.
			msg += tr("L7113:\n"
				"The ink cartridge cover is open.\n"
				"Close it.");
			break;

		case PP100API_HPR008:
			// Time to replace the maintenance box.
			msg += tr("L7601:\n"
				"It is time to replace the Maintenance Box. No further printing is possible.\n"
				"Please replace the Maintenance Box with a new one.\n"
				"\n"
				"Epson brand products are recommended.");
			break;

		case PP100API_HPR007:	// The maintenance box is not installed.
		case PP100API_HPR009:	// Cannot recognize the maintenance box.
			msg += tr("L7600:\n"
				"The Maintenance Box cannot be recognized.\n"
				"Set the Maintenance Box properly.\n"
				"\n"
				"Epson brand products are recommended.");
			break;

		case PP100API_HPR010:
			// The maintenance box cover is not placed correctly.
			msg += tr("L7602:\n"
				"The Maintenance Box cover is not set properly.\n"
				"Please check the cover.");
			break;

		case PP100API_HDR000:
			// Controlling the drive tray is failed.
			msg += tr("L7003:\n"
				"Opening and closing the drive tray failed.\n"
				"There may be a disc left inside the drive.\n"
				"Please turn off the output device and check inside the drive.\n"
				"If the same error occurs frequently, please contact support.");
			break;

		case PP100API_HDR001:
			// Processing the job is impossible because of a fatal error of the drive.
			msg += tr("L7013:\n"
				"An internal drive error occurred.\n"
				"Turn off the output device, unplug the power cable, and check inside the drive.\n"
				"If the same error occurs frequently, contact Customer Support.");
			break;

		case PP100API_HDR002:
			// The drive is not connected correctly.
			msg += tr("L7020:\n"
				"An error occured when communicating with the drive.\n"
				"Turn off the output device, unplug the power cable, and check the drive type and connection.\n"
				"If the same error occurs frequently, contact Customer Support.");
			break;

		case PP100API_HAR000:
			// Picking up a disc from the drive tray or the printer tray is failed.
			msg += tr("L7005:\n"
				"Disc pickup failed.\n"
				"There is a disc left in the drive or printer tray.\n"
				"Please turn off the output device and remove the disc from the tray.\n"
				"If the same error occurs frequently, please contact support.");
			break;

		case PP100API_HAR001:
			// A disc is dropped while being picked up.
			msg += tr("L7001:\n"
				"The arm dropped the disc while transferring it.\n"
				"Please turn off the output device and remove the disc from the equipment.\n"
				"If the same error occurs frequently,\n"
				"please contact support.");
			break;

		case PP100API_HAR002:
			// Releasing a disc is failed.
			msg += tr("L7006:\n"
				"Disc release failed.\n"
				"Please turn the output device off and then on again.\n"
				"If the same error occurs frequently, please contact support.");
			break;

		case PP100API_HAR003:
			// Setting a disc in the drive tray or the printer tray must have been made, however, no disc is detected in the tray.
			msg += tr("L7017:\n"
				"Detecting the transferred disc failed.\n"
				"Turn off the output device and remove the disc from the device.\n"
				"If the same error occurs frequently, contact Customer Support.");
			break;

		case PP100API_HAR004:
			// An error occurs while the robotic arm is moving.
			msg += tr("L7002:\n"
				"An error occurred while the arm was moving.\n"
				"There may be an obstruction in the path of the arm.\n"
				"Please turn off the output device and check inside the device.\n"
				"If the same error occurs frequently, please contact support.");
			break;

		case PP100API_HAR005:
			// An auto loader fatal error has occurred. The robotic arm can not be moved due to overheating of the motor.
			msg += tr("L7015:\n"
				"An internal auto loader error occurred.\n"
				"Turn off the output device, and check inside the device.\n"
				"If the same error occurs frequently, contact Customer Support.");
			break;

		case PP100API_HAR007:
			// Picking up a disc from the stacker is failed.
			msg += tr("L7114:\n"
				"Disc pickup failed.\n"
				"To ensure that discs are transferred properly, be sure to break the static attraction between discs in the stacker.\n"
				"If pickup fails after separating the discs, contact Customer Support.");
			break;

		case PP100API_HAR008:
			// Multiple discs have been transferred to the drive.
			msg += tr("L7104:\n"
				"Detected multiple discs transferred to the drive.\n"
				"Remove all discs from the drive tray to avoid causing any damage to the output device.\n"
				"Also, to ensure that discs are transferred properly, check that the discs are loosely arranged in the stacker.");
			break;

		case PP100API_HAR009:
			// Multiple discs have been transferred to the printer.
			msg += tr("L7105:\n"
				"Detected multiple discs transferred to the printer.\n"
				"Remove all discs from the printer tray to avoid causing any damage to the output device.\n"
				"Also, to ensure that discs are transferred properly, check that the discs are loosely arranged in the stacker.");
			break;

		case PP100API_HIKC000:	// The cyan ink cartridge has been removed.
		case PP100API_HIKC001:	// The model number of the cyan ink cartridge is different or the model number cannot be recognized.
			msg += tr("L7110:\n"
				"The following ink cartridge cannot be recognized.\n"
				"Set the ink cartridge correctly.\n"
				"\n"
				"Cyan (EPSON brand part number:PJIC1(C))\n"
				"Epson brand products are recommended.");
			break;

		case PP100API_HIKM000:	// The magenta ink cartridge has been removed.
		case PP100API_HIKM001:	// The model number of the magenta ink cartridge is different or the model number cannot be recognized.
			msg += tr("L7110:\n"
				"The following ink cartridge cannot be recognized.\n"
				"Set the ink cartridge correctly.\n"
				"\n"
				"Magenta (EPSON brand part number:PJIC4(M))\n"
				"Epson brand products are recommended.");
			break;

		case PP100API_HIKY000:	// The yellow ink cartridge has been removed.
		case PP100API_HIKY001:	// The model number of the yellow ink cartridge is different or the model number cannot be recognized.
			msg += tr("L7110:\n"
				"The following ink cartridge cannot be recognized.\n"
				"Set the ink cartridge correctly.\n"
				"\n"
				"Yellow (EPSON brand part number:PJIC5(Y))\n"
				"Epson brand products are recommended.");
			break;

		case PP100API_HIKLC000:	// The light cyan ink cartridge has been removed.
		case PP100API_HIKLC001:	// The model number of the light cyan ink cartridge is different or the model number cannot be recognized.
			msg += tr("L7110:\n"
				"The following ink cartridge cannot be recognized.\n"
				"Set the ink cartridge correctly.\n"
				"\n"
				"Light cyan (EPSON brand part number:PJIC2(LC))\n"
				"Epson brand products are recommended.");
			break;

		case PP100API_HIKLM000:	// The light magenta ink cartridge has been removed.
		case PP100API_HIKLM001:	// The model number of the light magenta ink cartridge is different or the model number cannot be recognized.
			msg += tr("L7110:\n"
				"The following ink cartridge cannot be recognized.\n"
				"Set the ink cartridge correctly.\n"
				"\n"
				"Light magenta (EPSON brand part number:PJIC3(LM))\n"
				"Epson brand products are recommended.");
			break;

		case PP100API_HIKBK000:	// The black ink cartridge has been removed.
		case PP100API_HIKBK001:	// The model number of the black ink cartridge is different or the model number cannot be recognized.
			msg += tr("L7110:\n"
				"The following ink cartridge cannot be recognized.\n"
				"Set the ink cartridge correctly.\n"
				"\n"
				"Black (EPSON brand part number:PJIC6(K))\n"
				"Epson brand products are recommended.");
			break;

		case PP100API_HST1000:
			// The number of discs in the stacker 1 has exceeded the upper limit.
			msg += tr("L7108：\n"
				"There are too many discs in Stacker 1.\n"
				"Please remove some discs.");
			break;

		case PP100API_HST1001:
			// The stacker 1 has not been installed.
			msg += tr("L7103:\n"
				"Stacker 1 is not set properly.\n"
				"Please check the stacker.");
			break;

		case PP100API_HST2000:
			// The number of discs in the stacker 2 has exceeded the upper limit.
			msg += tr("L7108：\n"
				"There are too many discs in Stacker 2.\n"
				"Please remove some discs.");
			break;

		case PP100API_HST2001:
			// The stacker 2 has not been installed.
			msg += tr("L7103:\n"
				"Stacker 2 is not set properly.\n"
				"Please check the stacker.");
			break;

		case PP100API_HST3000:
			// The number of discs in the stacker 3 has exceeded the upper limit.
			msg += tr("L7108：\n"
				"There are too many discs in Stacker 3.\n"
				"Please remove some discs.");
			break;

		case PP100API_HST3001:
			// The output stacker in Standard mode is set to stacker 3, but stacker 3 is not installed,
			// or the output stacker in External Output mode is set to stacker 3, but stacker 3 is not installed,
			// or stacker 3 is not installed in Batch mode.
			msg += tr("L7103:\n"
				"Stacker 3 is not set properly.\n"
				"Please check the stacker.");
			break;

		case PP100API_HST3002:
			// The output stacker in Standard mode is set to stacker 4, or the output stacker in External Output mode is set to stacker 4, but stacker 3 is installed.
			msg += tr("L7106:\n"
				"Stacker 3 is not necessary.\n"
				"Please remove Stacker 3.");
			break;

		case PP100API_HDC000:
			// The disc cover is open.
			msg += tr("L7102:\n"
				"The disc cover is open.\n"
				"Please close it.");
			break;

		case PP100API_HNW000:
			// Either one of the following communication errors occur.
			// (1) Communication with Disc producer is disabled.
			// (2) The filter driver has not been attached.
			msg += tr("L7000:\n"
				"An error occurred in communication with the output device.\n"
				"Please check the power supply of the output device and the connection.");
			break;

		case PP100API_HNW001:
			// A non-supported version error has occurred, and any of the following has been detected.
			// (1) Firmware version of the auto loader is not supported.
			// (2) Firmware version of the drive 1 is different from the supported version.
			// (3) Firmware version of the drive 2 is different from the supported version.
			// (4) Firmware version of the printer is not supported.
			msg += tr("L7008:\n"
				"The output device is an unsupported version.\n"
				"You cannot publish to this output device.\n"
				"For details, please contact support.");
			break;

		default:
			msg += tr("Unknown INFOMATION Code");
			break;
		}
	}

	if ( msg.isEmpty() )
	{
		for(const PP100_JOB_STATUS & jobStatus : jobStatusPublishing)
		{
			if( jobStatus.ulJobStatus == PP100API_JOB_PAUSED ||
				jobStatus.ulJobStatus == PP100API_JOB_WAITING ||
				jobStatus.ulJobStatus == PP100API_JOB_STANDBY )
			{
				switch(jobStatus.ulErrorCode)
				{
				case PP100API_STP000: msg += "PP100API_STP000"; break;
				case PP100API_STP001: msg += "PP100API_STP001"; break;
				case PP100API_STP002: msg += "PP100API_STP002"; break;
				case PP100API_RTN000: msg += "PP100API_RTN000"; break;
				case PP100API_RTN001: msg += "PP100API_RTN001"; break;
				case PP100API_RTN002: msg += "PP100API_RTN002"; break;
				case PP100API_RTN003: msg += "PP100API_RTN003"; break;
				case PP100API_RTN004: msg += "PP100API_RTN004"; break;
				case PP100API_RTN005: msg += "PP100API_RTN005"; break;
				case PP100API_RTN006: msg += "PP100API_RTN006"; break;
				case PP100API_RTN007: msg += "PP100API_RTN007"; break;
				case PP100API_RTN008: msg += "PP100API_RTN008"; break;
				case PP100API_RTN009: msg += "PP100API_RTN009"; break;
				case PP100API_RTN010: msg += "PP100API_RTN010"; break;
				case PP100API_RTN011: msg += "PP100API_RTN011"; break;
				case PP100API_RTN012: msg += "PP100API_RTN012"; break;
				case PP100API_RTN013: msg += "PP100API_RTN013"; break;
				case PP100API_RTN014: msg += "PP100API_RTN014"; break;
				case PP100API_RTN015: msg += "PP100API_RTN015"; break;
				case PP100API_RTN016: msg += "PP100API_RTN016"; break;
				case PP100API_RTN017: msg += "PP100API_RTN017"; break;
				default: msg += QString::asprintf("%lu", jobStatus.ulErrorCode); break;
				}
				break;
			}
		}
	}

	if ( msg.isEmpty() )
	{
		msg = tr("You can now publish discs.");
	}

	textEdit_DeviceStatus->setText(msg);
}

// Drive status setting
void TDMonitorEx::setDriveInfo(const PP100_PUBLISHER_STATUS& publisherStatus, int drvNum)
{
	QString msg;
	switch(publisherStatus.ulDriveStatus[drvNum])
	{
	case PP100API_UNUSED:
		msg = tr("Unused");
		break;
	case PP100API_TRANSFERRING:
		msg = tr("Transferring");
		break;
	case PP100API_PAUSED:
		msg = tr("Stopping");
		break;
	case PP100API_WRITING:
		msg = tr("Writing");
		break;
	case PP100API_VERIFYING_WRITE:
		msg = tr("Verifying Write");
		break;
	case PP100API_ERROR:
		msg = tr("Error");
		break;
	case PP100API_WAITING:
		msg = tr("Standby");
		break;
	default:
		fprintf(stderr, "unknown drive status: %lu\n", publisherStatus.ulDriveStatus[drvNum]);
		msg = tr("Unknown");
		break;
	}
	if(!drvNum)
	{
		label_Drive1Status->setText(msg);
		progressBar_Drive1->setValue(publisherStatus.ulDriveProgress[drvNum]);
		if(publisherStatus.ulDriveStatus[drvNum] == PP100API_VERIFYING_WRITE)
		{
			progressBar_Drive1->setStyleSheet(gDriveCompareStyle);
		}
		else if((publisherStatus.ulDriveStatus[drvNum] != PP100API_PAUSED) &&
				(publisherStatus.ulDriveStatus[drvNum] != PP100API_ERROR))
		{
			progressBar_Drive1->setStyleSheet(gDriveWriteStyle);
		}
	}
	else
	{
		label_Drive2Status->setText(msg);
		progressBar_Drive2->setValue(publisherStatus.ulDriveProgress[drvNum]);
		if(publisherStatus.ulDriveStatus[drvNum] == PP100API_VERIFYING_WRITE)
		{
			progressBar_Drive2->setStyleSheet(gDriveCompareStyle);
		}
		else if((publisherStatus.ulDriveStatus[drvNum] != PP100API_PAUSED) &&
				(publisherStatus.ulDriveStatus[drvNum] != PP100API_ERROR))
		{
			progressBar_Drive2->setStyleSheet(gDriveWriteStyle);
		}
	}
}

// Stacker status setting
void TDMonitorEx::setStackerSetting(const PP100_PUBLISHER_STATUS& publisherStatus, int stckerNum)
{
	QString msg;
	bool isUse = true;

	QLabel * labelStatus = NULL;
	QLabel * labelRest = NULL;
	switch(stckerNum)
	{
	case 0:		// Stacker1
		labelStatus = labelStacker1Status;
		labelRest = labelStacker1Rest;
		break;

	case 1:		// Stacker2
		labelStatus = labelStacker2Status;
		labelRest = labelStacker2Rest;
		break;

	case 2:		// Stacker3
		labelStatus = labelStacker3Status;
		labelRest = labelStacker3Rest;
		break;

	case 3:		// Stacker4
		labelStatus = labelStacker4Status;
		labelRest = labelStacker4Rest;
		break;

	default:
		assert(false);
		break;
	}

	if ( labelStatus != NULL && labelRest != NULL )
	{
		switch(publisherStatus.ulStackerSetting[stckerNum])
		{
		case PP100API_INPUT:
			labelStatus->setText(tr("Input Stacker"));
			break;

		case PP100API_OUTPUT:
			labelStatus->setText(tr("Output Stacker"));
			break;

		case PP100API_NOT_USE_STACKER:
			labelStatus->setText(tr("Unused"));
			isUse = false;
			break;

		case PP100API_DVD_MINUS_R:
		case PP100API_DVD_PLUS_R:
		case PP100API_DVD_R:
			labelStatus->setText(tr("DVD(+R/-R)"));
			break;

		case PP100API_DVD_MINUS_R_DL:
		case PP100API_DVD_PLUS_R_DL:
		case PP100API_DVD_R_DL:
			labelStatus->setText(tr("DVD(+R/-R) DL"));
			break;

		case PP100API_BD_MINUS_R:
			labelStatus->setText(tr("BD-R"));
			break;

		case PP100API_BD_MINUS_R_DL:
			labelStatus->setText(tr("BD-R DL"));
			break;

		case PP100API_CD_MINUS_R:
			labelStatus->setText(tr("CD-R"));
			break;

		default:
			fprintf(stderr, "unknown stacker setting: %lu\n", publisherStatus.ulStackerSetting[stckerNum]);
			labelStatus->setText(tr("Unknown"));
			break;
		}

		// Stacker Rest
		labelRest->setStyleSheet("");
		if(isUse)
		{
			switch(stckerNum)
			{
			case 0:		// Stacker1
			case 1:		// Stacker2
				switch( publisherStatus.ulStackerRest[stckerNum] )
				{
				case PP100API_ERROR:
					labelRest->setStyleSheet("background-color : rgb(255,96,0); color:white; font-weight:bold; border-radius:8px;");
					labelRest->setText("-");
					break;

				case PP100API_STACKER_NOT_FULL:
					labelRest->setText("-");
					break;

				case 100:
					labelRest->setStyleSheet("background-color : rgb(255,96,0); color:white; font-weight:bold; border-radius:8px;");
					labelRest->setText("100 %");
					break;

				default:
					labelRest->setText(QString::asprintf("%lu %%", publisherStatus.ulStackerRest[stckerNum]));
					break;
				}
				break;

			case 2:		// Stacker3
			case 3:		// Stacker4
				switch( publisherStatus.ulStackerRest[stckerNum] )
				{
				case 100:
					labelRest->setStyleSheet("background-color : rgb(255,96,0); color:white; font-weight:bold; border-radius:8px;");
					labelRest->setText("100 %");
					break;

				case 0:
					labelRest->setText("0 %");
					break;

				default:
					labelRest->setText("-");
				}
				break;

			default:
				assert(false);
				break;
			}
		}
		else
		{
			labelRest->setText("-");
		}
	}
}

// JOB cancel processing
void TDMonitorEx::cancelJob()
{
	// Get selected item.
	std::vector<std::string> jobIdList;
	for(auto item : treeJobList->selectedItems())
	{
		jobIdList.push_back(item->text(0).toStdString());
	}

	if (!jobIdList.empty())
	{
		for(auto jobId : jobIdList)
		{
			char pJobID[41] = {0};
			strcpy(pJobID, jobId.c_str());
			long ret = g_Api.PP100_CancelJob(g_Api.ulHandle, pJobID);
			if(PP100API_SUCCESS != ret)
			{
				// Cancel until the end even if there is an error.
				printf("cancelJob Faild JobID = %s, ret = %ld\n", pJobID, ret);
			}
		}
		updateContent();
		updateMenu();
		m_timer->start();
	}
}

// JOB all selection processing
void TDMonitorEx::selectAllJobs()
{
	for(int nCnt = 0; nCnt < treeJobList->topLevelItemCount(); ++nCnt)
	{
		QTreeWidgetItem *item = treeJobList->topLevelItem(nCnt);
		item->setSelected(true);
	}
}

// Unfinished JOB tab processing
void TDMonitorEx::showUnfinishedJobs()
{
	tabWidget->setCurrentIndex(0);
	QTreeWidgetItem *item = treeJobList->headerItem();
	item->setText(6, tr("Time Needed"));
	item->setText(5, tr("Output Stacker"));
	item->setText(4, tr("Input Stacker"));
	item->setText(3, tr("Type"));
	item->setText(2, tr("Copies"));
	treeJobList->setContextMenuPolicy(Qt::ActionsContextMenu);
	actionShowUnfinishedJobs->setChecked(true);
	updateContent();
	updateMenu();
	m_timer->start();
}

// Finished JOB tab processing
void TDMonitorEx::showFinishedJobs()
{
	tabWidget->setCurrentIndex(1);
	QTreeWidgetItem *item = treeJobList->headerItem();
	item->setText(6, tr("Output Stacker"));
	item->setText(5, tr("Type"));
	item->setText(4, tr("Quantity"));
	item->setText(3, tr("Failed"));
	item->setText(2, tr("Successful"));
	treeJobList->setContextMenuPolicy(Qt::PreventContextMenu);
	actionShowFinishedJobs->setChecked(true);
	updateContent();
	updateMenu();
	m_timer->start();
}

// Display version information dialog
void TDMonitorEx::showAboutDialog()
{
	QDialog *dialog = new QDialog;
	Ui::AboutDialog ui;
	ui.setupUi(dialog);
	dialog->setModal(true);
	dialog->show();
	dialog->exec();
}

// Tab switching process
void TDMonitorEx::changeTab(int num)
{
	if(!num)
	{
		showUnfinishedJobs();
	}
	else
	{
		showFinishedJobs();
	}
}

// Screen update process
void TDMonitorEx::updatePeriodical()
{
	std::vector<std::string> jobIdList;
	for(auto item : treeJobList->selectedItems())
	{
		jobIdList.push_back(item->text(0).toStdString());
	}

	updateContent();

	if (!jobIdList.empty())
	{
		for (int i=0; i < treeJobList->topLevelItemCount(); i++)
		{
			QTreeWidgetItem * item = treeJobList->topLevelItem(i);
			for(auto jobId : jobIdList)
			{
				if ( item->text(0).toStdString() == jobId )
				{
					item->setSelected(true);
					break;
				}
			}
		}
	}
	updateMenu();
}

// Screen update cycle setting (normal time: 5000msec, window minimization: stop)
bool TDMonitorEx::event(QEvent* event)
{
	if(event->type() == QEvent::WindowStateChange)
	{
		QWindowStateChangeEvent* ev = static_cast<QWindowStateChangeEvent*>(event);
		if(!(ev->oldState() & Qt::WindowMinimized) && (windowState() & Qt::WindowMinimized))
		{
			m_timer->stop();
		}
		else if((ev->oldState() & Qt::WindowMinimized) && !(windowState() & Qt::WindowMinimized))
		{
			m_timer->setInterval(5000);
			updateContent();
			updateMenu();
			m_timer->start();
		}
	}
	return QMainWindow::event(event);
}

// Menu enable / disable update
void TDMonitorEx::updateMenu()
{
	if(treeJobList->topLevelItemCount() != 0)
	{
		QList<QTreeWidgetItem *> itemList = treeJobList->selectedItems();
		if(itemList.count() == 0)
		{
			actionCancelJob->setEnabled(false);
		}
		else
		{
			actionCancelJob->setEnabled(true);
		}
		actionSelectAllJobs->setEnabled(true);
	}
	else
	{
		actionCancelJob->setEnabled(false);
		actionSelectAllJobs->setEnabled(false);
	}
}
